@@ -2,11 +2,15 @@
 # -*- coding: utf-8 -*-
 # This file is part of the covid-surge application.
 # https://github/dpploy/covid-surge
+# Valmor F. de Almeida dealmeidavf@gmail.com
 import os
 import logging
 import time
 import datetime
+
 import numpy as np
+import matplotlib
+import matplotlib.pyplot as plt
 
 class Surge:
 
@@ -22,8 +26,8 @@ class Surge:
 
         self.deaths_100k_minimum = 40 # US death per 100,000 for Chronic Lower Respiratory Diseases per year: 41 (2019)
 
-        self.state_names = list()
-        self.populations = list()
+        self.state_names   = list()
+        self.populations   = None
         self.country_names = list()
 
         if self.locale == 'US':
@@ -251,12 +255,9 @@ class Surge:
 
     def plot_covid_data(self, name, save=False):
 
-        import matplotlib.pyplot as plt
-        plt.rcParams['figure.figsize'] = [12, 5]
-
         population = None
 
-        if name == 'US':
+        if name == 'US' and self.locale == 'US':
             # Combine all column data in the surge
             cases_plot = np.sum(self.cases,axis=1)
             population = np.sum(self.populations)
@@ -293,7 +294,8 @@ class Surge:
 
         source = 'Johns Hopkins CSSE: https://github.com/CSSEGISandData/COVID-19'
 
-        fig, ax = plt.subplots(figsize=(20,6))
+        fig, ax = plt.subplots(figsize=(15,6))
+        #plt.rcParams['figure.figsize'] = [12, 5]
 
         ax.plot( range(len(dates_plot)), cases_plot, 'r*', label=source )
 
@@ -320,22 +322,18 @@ class Surge:
 
         plt.show()
         if save:
-            plt.savefig('covid_data_'+filename+'.png', dpi=100)
+            plt.savefig('data_'+filename+'.png', dpi=100)
         plt.close()
 
         return
 
     def fit_data(self, name ):
 
-        population = None
-
-        if name == 'US':
+        if name == 'US' and self.locale == 'US':
             # Combine all column data in the surge
             cases      = np.sum(self.cases,axis=1)
-            population = np.sum(self.populations)
         elif name in self.state_names:
             state_id = self.state_names.index(name)
-            population = self.populations[state_id]
             cases = self.cases[:,state_id]
         elif name in self.country_names:
             country_id = self.country_names.index(name)
@@ -386,11 +384,9 @@ class Surge:
 
         formula = self.sigmoid_formula
 
-        import matplotlib.pyplot as plt
-
         population = None
 
-        if name == 'US':
+        if name == 'US' and self.locale == 'US':
             # Combine all column data in the surge
             cases_plot = np.sum(self.cases,axis=1)
             population = np.sum(self.populations)
@@ -425,8 +421,7 @@ class Surge:
 
         source = 'Johns Hopkins CSSE: https://github.com/CSSEGISandData/COVID-19'
 
-        plt.figure(1)
-        plt.rcParams['figure.figsize'] = [12, 5]
+        plt.figure(1,figsize=(15,5))
 
         if option == 'dates':
             plt.plot(dates_plot, cases_plot,'r*',label=source)
@@ -543,7 +538,7 @@ class Surge:
 
         plt.show()
         if save:
-            plt.savefig('covid_data_fit_'+filename+'_0'+'.png', dpi=100)
+            plt.savefig('data_fit_'+filename+'_0'+'.png', dpi=100)
 
         plt.close()
 
@@ -553,8 +548,7 @@ class Surge:
 
             fit_func_prime = self.__sigmoid_func_prime
 
-            plt.figure(2)
-            plt.rcParams['figure.figsize'] = [12, 5]
+            plt.figure(2,figsize=(15,5))
 
             n_rows = 1
             n_cols = 1
@@ -609,7 +603,7 @@ class Surge:
 
             plt.show()
             if save:
-                plt.savefig('covid_data_fit_'+filename+'_1'+'.png', dpi=100)
+                plt.savefig('fit_'+filename+'_1'+'.png', dpi=100)
             plt.close()
 
         # Additional plot for second derivative 
@@ -617,8 +611,7 @@ class Surge:
 
             fit_func_double_prime = self.__sigmoid_func_double_prime
 
-            plt.figure(3)
-            plt.rcParams['figure.figsize'] = [12, 5]
+            plt.figure(3,figsize=(15,5))
 
             n_rows = 1
             n_cols = 1
@@ -682,7 +675,7 @@ class Surge:
 
             plt.show()
             if save:
-                plt.savefig('covid_data_fit_'+filename+'_2'+'.png', dpi=100)
+                plt.savefig('fit_'+filename+'_2'+'.png', dpi=100)
             plt.close()
 
         return
@@ -873,7 +866,7 @@ class Surge:
 
         import math
 
-        if name == 'US':
+        if name == 'US' and self.locale == 'US':
             # Combine all column data in the surge
             cases = np.sum(self.cases,axis=1)
         elif name in self.state_names:
@@ -990,7 +983,7 @@ class Surge:
 
         population = None
 
-        if name == 'US':
+        if name == 'US' and self.locale == 'US':
             # Combine all column data in the surge
             cases = np.sum(self.cases,axis=1)
         elif name in self.state_names:
@@ -1058,76 +1051,91 @@ class Surge:
 
         return
 
-    def states_fit_data(self, verbose=False, plot=False, save_plots=False):
+    def multi_fit_data(self, name, 
+            blocked_list=[],
+            verbose=False, plot=False, save_plots=False):
+
+        if name == 'states':
+            names = self.state_names
+        elif name == 'countries':
+            names = self.country_names
+        else:
+            assert False,'Bad name %r (states or countries)'%name
+
+        cases = self.cases
 
         # Sort the states by descending number of total cases
-        sorted_states = sorted(
-                zip( self.state_names, self.cases[-1,:] ),
-                key = lambda entry: entry[1], reverse=True
-                          )
+        sorted_list = sorted(
+               zip( names, cases[-1,:] ),
+               key = lambda entry: entry[1], reverse=True
+                             )
 
         # Post processing data storage
         fit_data = list()
-        states_past_peak_surge_period = list()
-        states_no_peak_surge_period = list()
-        states_below_deaths_100k_minimum = list()
-        states_below_deaths_abs_minimum = list()
+        names_past_peak_surge_period = list()
+        names_no_peak_surge_period = list()
+        names_below_deaths_100k_minimum = list()
+        names_below_deaths_abs_minimum = list()
 
         top_id = 0
 
-        for (state,dummy) in sorted_states:
+        for (name,dummy) in sorted_list:
 
-            assert state in self.state_names, 'State: %r not in %r'%(state,self.state_names)
-            state_id = self.state_names.index(state)
-            population = self.populations[state_id]
-            cases = self.cases[:,state_id]
+            if name in blocked_list:
+                continue
 
+            assert name in names, 'Name: %r not in %r'%(name,names)
+            name_id = names.index(name)
+            if self.populations:
+                population = self.populations[name_id]
+            icases = cases[:,name_id]
 
-            if cases[-1] < self.min_n_cases_abs:
+            if icases[-1] < self.min_n_cases_abs:
                 if verbose:
                     print('')
-                    print('WARNING: state %r # deaths: %r below absolute minimum'%(state,cases[-1]))
+                    print('WARNING: name %r # deaths: %r below absolute minimum'%(name,icases[-1]))
                     print('')
-                states_below_deaths_abs_minimum.append((state,cases[-1]))
+                names_below_deaths_abs_minimum.append((name,icases[-1]))
                 continue
 
             # Select data with # of cases greater than the minimum
-            (nz_cases_ids,) = np.where(cases>self.min_n_cases_rel/100*cases[-1])
+            (nz_cases_ids,) = np.where(icases>self.min_n_cases_rel/100*icases[-1])
 
             if nz_cases_ids.size == 0:
                 if verbose:
                     print('')
-                    print('WARNING: No data for state %r. Continuing...'%state)
+                    print('WARNING: No data for state %r. Continuing...'%name)
                     print('')
                 continue
 
-            cases = np.copy(cases[nz_cases_ids])
+            icases = np.copy(icases[nz_cases_ids])
             dates = self.dates[nz_cases_ids]
 
-            deaths_100k = round(cases[-1]*100000/population * 365/dates.size,1)
+            if self.populations:
+                deaths_100k = round(icases[-1]*100000/population * 365/dates.size,1)
 
-            if deaths_100k < self.deaths_100k_minimum:
-                if verbose:
-                    print('')
-                    print('WARNING: state %r deaths per 100k: %r below minimum'%(state,deaths_100k))
-                    print('')
-                states_below_deaths_100k_minimum.append((state,deaths_100k))
-                continue
+                if deaths_100k < self.deaths_100k_minimum:
+                    if verbose:
+                        print('')
+                        print('WARNING: name %r deaths per 100k: %r below minimum'%(name,deaths_100k))
+                        print('')
+                    names_below_deaths_100k_minimum.append((name,deaths_100k))
+                    continue
 
             if verbose:
                 print('')
                 print('********************************************************')
-                print('                     '+state)
+                print('                     '+name)
                 print('********************************************************')
                 print('')
 
-            scaling = cases.max()
-            cases /= scaling
+            scaling = icases.max()
+            icases /= scaling
 
-            a0 = cases[-1]
-            a1 = a0/cases[0] - 1
+            a0 = icases[-1]
+            a1 = a0/icases[0] - 1
             a2 = -0.15
-            if state == 'Michigan':
+            if name == 'Michigan':
                 a2 = -.1
 
             param_vec_0 = np.array([a0,a1,a2])
@@ -1137,7 +1145,7 @@ class Surge:
             k_max = 25
             rel_tol = 0.01 / 100.0 # (0.1%)
 
-            (param_vec,r2,k) = self.__newton_nlls_solve( times, cases,
+            (param_vec,r2,k) = self.__newton_nlls_solve( times, icases,
                                self.sigmoid_func, self.__grad_p_sigmoid_func,
                                param_vec_0, k_max, rel_tol, verbose=False )
 
@@ -1155,7 +1163,7 @@ class Surge:
             assert param_vec[2] < 0.0
 
             param_vec[0] *= scaling
-            cases  *= scaling
+            icases  *= scaling
 
             if verbose:
                 print('')
@@ -1163,14 +1171,14 @@ class Surge:
                 print('')
 
             # Compute critical times
-            (tc,dtc) = self.critical_times(state,param_vec,verbose=verbose)
+            (tc,dtc) = self.critical_times(name,param_vec,verbose=verbose)
 
             if tc > times[-1]:
                 if verbose:
                     print('')
                     print('WARNING: Time at peak surge rate exceeds time data.')
                     print('WARNING: Skipping this data set.')
-                states_no_peak_surge_period.append( (state, tc-dtc, dates[int(tc-dtc)+1], dtc) )
+                names_no_peak_surge_period.append( (name, tc-dtc, dates[int(tc-dtc)+1], dtc) )
                 continue
 
             if tc + dtc > times[-1]:
@@ -1178,7 +1186,7 @@ class Surge:
                     print('')
                     print('WARNING: Time at mininum acceleration exceeds time data.')
                     print('WARNING: Skipping this data set.')
-                states_past_peak_surge_period.append( (state, tc, dates[int(tc)+1], dtc) )
+                names_past_peak_surge_period.append( (name, tc, dates[int(tc)+1], dtc) )
                 continue
 
 
@@ -1186,24 +1194,24 @@ class Surge:
 
 
             if plot:
-                self.plot_covid_nlfit( state, param_vec, save=save_plots )
+                self.plot_covid_nlfit( name, param_vec, save=save_plots )
 
 
             n_last_days = 7
             if verbose:
                 print('')
                 print('Last %i days'%n_last_days,
-                      ' # of cumulative cases = ',cases[-n_last_days:])
+                      ' # of cumulative cases = ',icases[-n_last_days:])
                 print('Last %i days'%n_last_days,
                       ' # of added cases =',
-                      [round(b-a,0) for (b,a) in zip( cases[-(n_last_days-1):],
-                                                      cases[-n_last_days:-1])
+                      [round(b-a,0) for (b,a) in zip( icases[-(n_last_days-1):],
+                                                      icases[-n_last_days:-1])
                     ])
                 print('')
 
             # Report erros
             if verbose:
-                self.error_analysis(state, param_vec, tc, dtc)
+                self.error_analysis(name, param_vec, tc, dtc)
 
             # 60-day look-ahead
             n_prediction_days = 60
@@ -1214,44 +1222,44 @@ class Surge:
             if verbose:
                 print('')
                 print('Estimated cumulative deaths in %s days from %s = %6i'%(n_prediction_days,dates[-1],total_deaths_predicted))
-                print('# of cumulative deaths today, %s               = %6i'%(dates[-1],cases[-1]))
+                print('# of cumulative deaths today, %s               = %6i'%(dates[-1],icases[-1]))
                 print('')
 
 
-            fit_data.append( [ state,
+            fit_data.append( [ name,
                                dates,
-                               cases,
+                               icases,
                                param_vec,
                                tc,
                                dtc] )
 
 
         if verbose:
-            print('States with significant deaths past peak in surge period:')
+            print('Names with significant deaths past peak in surge period:')
             print('')
-            for (state, tc, tc_date, dtc) in states_past_peak_surge_period:
-                print( '%20s tc = %3.1f [d] tc_date = %8s pending days = %3.1f'%(state,tc,tc_date,dtc))
+            for (name, tc, tc_date, dtc) in names_past_peak_surge_period:
+                print( '%20s tc = %3.1f [d] tc_date = %8s pending days = %3.1f'%(name,tc,tc_date,dtc))
 
             print('')
 
-            print('States with significant deaths before peak in surge period:')
+            print('Names with significant deaths before peak in surge period:')
             print('')
-            for (state, tc_minus_dtc, tc_minus_dtc_date, dtc) in states_no_peak_surge_period:
-                print( '%15s tc-dtc = %3.1f [d] tc-dtc_date = %8s pending days = %3.1f'%(state,tc_minus_dtc,tc_minus_dtc_date,dtc))
+            for (name, tc_minus_dtc, tc_minus_dtc_date, dtc) in names_no_peak_surge_period:
+                print( '%15s tc-dtc = %3.1f [d] tc-dtc_date = %8s pending days = %3.1f'%(name,tc_minus_dtc,tc_minus_dtc_date,dtc))
 
             print('')
 
-            print('States with deaths per 100k below mininum:')
+            print('Names with deaths per 100k below mininum:')
             print('')
-            for (state, deaths_100k) in states_below_deaths_100k_minimum:
-                print( '%15s deaths per 100k/y = %5.2f'%(state,deaths_100k))
+            for (name, deaths_100k) in names_below_deaths_100k_minimum:
+                print( '%15s deaths per 100k/y = %5.2f'%(name,deaths_100k))
 
             print('')
 
             print('States with deaths below the absolute mininum:')
             print('')
-            for (state, case) in states_below_deaths_abs_minimum:
-                print( '%15s deaths = %5.2f'%(state,case) )
+            for (name, case) in names_below_deaths_abs_minimum:
+                print( '%15s deaths = %5.2f'%(name,case) )
 
         # Order fit_data 
 
@@ -1267,10 +1275,8 @@ class Surge:
 
         return sorted_fit_data
 
-    def plot_fit_data(self, fit_data, option=None, save=False):
+    def plot_multi_fit_data(self, fit_data, option=None, save=False):
 
-        import matplotlib
-        import matplotlib.pyplot as plt
         from covid_surge import color_map
 
         if option == 'experimental':
@@ -1278,9 +1284,10 @@ class Surge:
             legend_title = 'Max. Relative Death Rate [%/day]'
             legend_title = 'Surge Period [day]'
 
-            fig, ax1 = plt.subplots(1, figsize=(20, 8))
+            fig, ax1 = plt.subplots(1, figsize=(15, 6))
 
             colors = color_map(len(fit_data))
+
             for (sort_key,data) in fit_data:
                 color = colors[fit_data.index((sort_key,data))]
                 state = data[0]
@@ -1301,12 +1308,20 @@ class Surge:
                 ax1.legend(loc='best',fontsize=12,title=legend_title)
 
             ax1.grid(True)
+
+            (key,data)=fit_data[0]
+            if data[0] in self.state_names:
+                data_name = 'US States'
+            if data[0] in self.country_names:
+                data_name = 'Countries'
+
             plt.title('COVID-19 Pandemic 2020 for Top '+
-                str(len(fit_data))+' US States in Total Mortality ('+
+                str(len(fit_data))+' '+data_name+' in Evolved Mortality ('+
                 data[1][-1]+')',fontsize=20)
+
             plt.show()
             if save:
-                plt.savefig('covid_data_fit_overlap'+'_0'+'.png', dpi=100)
+                plt.savefig('data_overlap'+'_0'+'.png', dpi=100)
             plt.close()
 
 
@@ -1315,9 +1330,10 @@ class Surge:
             legend_title = 'Max. Relative Death Rate [%/day]'
             legend_title = 'Surge Period [day]'
 
-            fig, ax1 = plt.subplots(1, figsize=(20, 8))
+            fig, ax1 = plt.subplots(1, figsize=(15, 6))
 
             colors = color_map(len(fit_data))
+
             for (sort_key,data) in fit_data:
                 color = colors[fit_data.index((sort_key,data))]
                 state = data[0]
@@ -1329,7 +1345,7 @@ class Surge:
                 t2 = tshift + data[5]
                 value = '%1.1f'%sort_key
 
-                ax1.plot(np.array(range(n_dates))-tshift, self.sigmoid_func(np.array(range(n_dates)),param_vec)/param_vec[0],
+                ax1.plot( np.array(range(n_dates))-tshift, self.sigmoid_func(np.array(range(n_dates)),param_vec)/param_vec[0],
                      'b-',label=state+': '+value,color=color)
 
                 ax1.plot(t1-tshift,self.sigmoid_func(t1,param_vec)/param_vec[0],'*',color=color,markersize=12)
@@ -1344,10 +1360,20 @@ class Surge:
                 ax1.legend(loc='best',fontsize=12,title=legend_title)
 
             ax1.grid(True)
-            plt.title('COVID-19 Pandemic 2020 for Top '+str(len(fit_data))+' US States ('+data[1][-1]+')',fontsize=20)
+
+            (key,data)=fit_data[0]
+            if data[0] in self.state_names:
+                data_name = 'US States'
+            if data[0] in self.country_names:
+                data_name = 'Countries'
+
+            plt.title('COVID-19 Pandemic 2020 for Top '+
+                str(len(fit_data))+' '+data_name+' in Evolved Mortality ('+
+                data[1][-1]+')',fontsize=20)
+
             plt.show()
             if save:
-                plt.savefig('covid_data_fit_overlap'+'_1'+'.png', dpi=100)
+                plt.savefig('fit_overlap'+'_1'+'.png', dpi=100)
             plt.close()
 
         return
@@ -1395,8 +1421,6 @@ class Surge:
         Plot fit functions for each country group
         '''
 
-        import matplotlib
-        import matplotlib.pyplot as plt
         from covid_surge import color_map
 
         legend_title = 'Max. Relative Death Rate [%/day]'
@@ -1444,22 +1468,31 @@ class Surge:
                 ax1.legend(loc='best',fontsize=16,title=legend_title,title_fontsize=18)
             else:
                 ax1.legend(loc='best',fontsize=16,title=legend_title)
+
             ax1.grid(True)
-            plt.title('COVID-19 Pandemic 2020 for Top '+str(len(fit_data))+' US States ('+data[1][-1]+')',fontsize=20)
+
+            (key,data)=fit_data[0]
+            if data[0] in self.state_names:
+                data_name = 'US States'
+            if data[0] in self.country_names:
+                data_name = 'Countries'
+
+            plt.title('COVID-19 Pandemic 2020 for Top '+
+                str(len(fit_data))+' '+data_name+' in Evolved Mortality ('+
+                data[1][-1]+')',fontsize=20)
 
             plt.show()
             if save:
-                plt.savefig('covid_data_states_group_'+str(ig)+'.png', dpi=100)
+                plt.savefig('fit_group_'+str(ig)+'.png', dpi=100)
             plt.close()
 
         return
 
     def plot_group_surge_periods(self, fit_data, bins, save=False):
 
-        import matplotlib.pyplot as plt
         from covid_surge import color_map
 
-        plt.rcParams['figure.figsize'] = [20, 4]
+        #plt.rcParams['figure.figsize'] = [20, 4]
         fig, ax = plt.subplots(figsize=(20,6))
 
         surge_periods = list()
@@ -1483,7 +1516,13 @@ class Surge:
             color = colors[ self.get_bin_id(val,bins) ]
             ax.bar( id, val, color=color )
 
+        # Fine tunning the axes
         ax.set_xlim((-.75,len(fit_data)))
+
+        # make space for annotation
+        (ymin,ymax)= ax.get_ylim()
+        ax.set_ylim((ymin,ymax+(ymax-ymin)*0.15))
+
         (xmin,xmax)= ax.get_xlim()
         (ymin,ymax)= ax.get_ylim()
 
@@ -1493,9 +1532,15 @@ class Surge:
             if group_id == len(bins.keys())-1:
                 ax.plot((-.75,len(fit_data)), [b[1],b[1]], 'k-.',linewidth=0.75 )
 
-        #label='mean: %2.1f; std: %2.1f (%2.1f %%)'%\
-        #                (mean,std,std/mean*100))
-        #ax.plot( )
+        # Annotation
+        annot='mean: %2.1f; std: %2.1f (%2.1f %%)'%(mean,std,std/mean*100)
+        dx = abs(xmax-xmin)
+        xtext = xmin + dx*0.02
+
+        dy = abs(ymax-ymin)
+        ytext = ymin + dy*0.90
+
+        ax.text(xtext, ytext, annot, fontsize=16)
 
         plt.xticks( range(len(states)), [state for (state,val) in sorted_list],
                 rotation=80,fontsize=16)
@@ -1504,12 +1549,21 @@ class Surge:
         ax.set_ylabel('Surge Period [day]',fontsize=16)
         ax.set_xlabel('',fontsize=20)
         ax.xaxis.grid(True,linestyle='-',which='major',color='lightgrey',alpha=0.9)
-        #plt.legend(loc='best',fontsize=14)
-        plt.title('COVID-19 Pandemic 2020 for Top '+str(len(fit_data))+' US States ('+data[1][-1]+')',fontsize=20)
+        (key,data)=fit_data[0]
+        if data[0] in self.state_names:
+            data_name = 'US States'
+        if data[0] in self.country_names:
+            data_name = 'Countries'
+
+        plt.title('COVID-19 Pandemic 2020 for Top '+
+            str(len(fit_data))+' '+data_name+' in Evolved Mortality ('+
+            data[1][-1]+')',fontsize=20)
+
         plt.tight_layout(1)
+
         plt.show()
         if save:
-            plt.savefig('covid_group_surge_periods.png', dpi=100)
+            plt.savefig('group_surge_periods.png', dpi=100)
         plt.close()
 
         return
